"""
Module - dr.botzo modular functionality base class
Copyright (C) 2010  Brian S. Stephan

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from ConfigParser import NoSectionError, NoOptionError

import inspect
import re
import sys
import sqlite3

from extlib import irclib

class Module(object):
    """Declare a base class used for creating classes that have real functionality."""

    def priority(self):
        return 50

    def __init__(self, irc, config, server):
        """Construct a feature module. Inheritors can do special things here, but
        should be sure to call Module.__init__.
        """

        self.irc = irc
        self.config = config
        self.server = server

        # open database connection
        dbfile = self.config.get('dr.botzo', 'database')
        self.conn = sqlite3.connect(dbfile)
        self.conn.row_factory = sqlite3.Row

        # setup regexp function in sqlite
        def regexp(expr, item):
            reg = re.compile(expr, re.IGNORECASE)
            return reg.search(item) is not None
        self.conn.create_function('REGEXP', 2, regexp)

        # set up database for this module
        self.db_init()

        # print what was loaded, for debugging
        print("loaded " + self.__class__.__name__)

    def register_handlers(self, server):
        """Hook handler functions into the IRC library. This is called when the
        module is loaded. Classes with special stuff to do could implement this
        and set up the appropriate handlers, e.g.:

        self.server.add_global_handler('welcome', self.on_connect)

        Unless you're sure what you're doing, don't register pubmsg or privmsg.
        They are handled by the alias support in DrBotIRC, and will call your
        module's do().
        """

    def unregister_handlers(self):
        """Unhook handler functions from the IRC library. Inverse of the above.
        This is called by reload, to remove the soon-to-be old object from the server
        global handlers (or whatever has been added via register_handlers). Classes
        inheriting from Module could implement this, e.g.:

        self.server.remove_global_handler('welcome', self.on_connect)
        """

    def save(self):
        """Save whatever the module may need to save. Sync files, etc.

        Implement this if you need it.
        """

    def shutdown(self):
        """Do pre-deletion type cleanup.

        Implement this to close databases, write to disk, etc. Note that DrBotIRC
        will call save() before this, so implement appropriately.
        """

    def remove_metaoptions(self, list):
        """Remove metaoptions from provided list, which was probably from a config file."""

        list.remove('debug')

    def retransmit_event(self, event):
        """
        Pretend that some event the bot has generated is rather an incoming IRC
        event. Why one would do this is unclear, but I wrote it and then realized
        I didn't need it.
        """

        self.server._handle_event(event)

    def get_db(self):
        """
        Get a database connection to sqlite3. Once grabbed, it should be closed
        when work is done. Modules that need a database connection should
        test for and create (or, eventually, alter) required table structure
        in their __init__ IF that structure does not already exist. Well-behaved
        modules should use a prefix in their table names (eg "karma_log" rather
        than "log")

        See also db_module_registered, below.
        """

        return self.conn

    def db_module_registered(self, modulename):
        """
        ask the database for a version number for a module. Return that version
        number if the module has registered, or None if not
        """

        conn = self.get_db()
        version = None
        try:
            cur = conn.cursor()
            cur.execute("SELECT version FROM drbotzo_modules WHERE module = :name",
                {'name': modulename})
            version = cur.fetchone()
            if (version != None):
                version = version[0]
        except sqlite3.Error as e:
            print("sqlite error:" + str(e))

        return version

    def db_init(self):
        """
        Set up the database tables and so on, if subclass is planning on using it.
        """

    def do(self, connection, event, nick, userhost, what, admin_unlocked):
        """Do the primary thing this module was intended to do, in most cases.

        Implement this method in your subclass to have a fairly-automatic hook into
        IRC functionality. This is called by DrBotIRC during pubmsg and privmsg
        events.
        """

        print("looks like someone forgot to implement do!")

# vi:tabstop=4:expandtab:autoindent
# kate: indent-mode python;indent-width 4;replace-tabs on;
